/*  ===========================================================================

BSD License

For Zstandard software

Copyright (c) 2016-present, Facebook, Inc. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

* Neither the name Facebook nor the names of its contributors may be used to
endorse or promote products derived from this software without specific
prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*   ===========================================================================
*/

#ifndef ZSTD_OUTPUT_STREAM_H_INCLUDED
#define ZSTD_OUTPUT_STREAM_H_INCLUDED

namespace zstd {
using namespace juce;

/** An OutputStream that wraps another OutputStream and compresses it using zstd.
*
*	Note: this is not the fastest interface to the zstd algorithms.
*	      If performance is important, use one of the other classes
*/
class ZstdOutputStream : public OutputStream
{
public:

	// =========================================================================

	ZstdOutputStream(OutputStream* ownedOutputStream, int compressionLevel = 5);
	~ZstdOutputStream();

	// =========================================================================

	void flush() override;
	bool setPosition(int64 newPosition) override;
	int64 getPosition() override;
	bool write(const void* dataToWrite, size_t numberOfBytes) override;

	// =========================================================================

private:

	struct Pimpl;
	ScopedPointer<Pimpl> pimpl;

	// =========================================================================

	JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(ZstdOutputStream);
};

}

#endif